@@ -143,6 +143,25 @@ var gBrowserInit = {
     gNavToolbox.palette = document.getElementById(
       "BrowserToolbarPalette"
     ).content;
+
+    let isVerticalTabs = Services.prefs.getBoolPref(
+      "sidebar.verticalTabs",
+      false
+    );
+    let nonRemovables;
+
+    // We don't want these normally non-removable elements to get put back into the
+    // tabstrip if we're initializing with vertical tabs.
+    // We should refrain from excluding popups here to make sure CUI doesn't
+    // get into a blank saved state.
+    if (isVerticalTabs) {
+      nonRemovables = [gBrowser.tabContainer];
+      for (let elem of nonRemovables) {
+        elem.setAttribute("removable", "true");
+        // tell CUI to ignore this element when it builds the toolbar areas
+        elem.setAttribute("skipintoolbarset", "true");
+      }
+    }
     for (let area of CustomizableUI.areas) {
       let type = CustomizableUI.getAreaType(area);
       if (type == CustomizableUI.TYPE_TOOLBAR) {
@@ -150,6 +169,35 @@ var gBrowserInit = {
         CustomizableUI.registerToolbarNode(node);
       }
     }
+    if (isVerticalTabs) {
+      // Show the vertical tabs toolbar
+      setToolbarVisibility(
+        document.getElementById(CustomizableUI.AREA_VERTICAL_TABSTRIP),
+        true,
+        false,
+        false
+      );
+      let tabstripToolbar = document.getElementById(
+        CustomizableUI.AREA_TABSTRIP
+      );
+      let wasCollapsed = tabstripToolbar.collapsed;
+      TabBarVisibility.update();
+      if (tabstripToolbar.collapsed !== wasCollapsed) {
+        let eventParams = {
+          detail: {
+            visible: !tabstripToolbar.collapsed,
+          },
+          bubbles: true,
+        };
+        let event = new CustomEvent("toolbarvisibilitychange", eventParams);
+        tabstripToolbar.dispatchEvent(event);
+      }
+
+      for (let elem of nonRemovables) {
+        elem.setAttribute("removable", "false");
+        elem.removeAttribute("skipintoolbarset");
+      }
+    }
     BrowserSearch.initPlaceHolder();
 
     // Hack to ensure that the various initial pages favicon is loaded
@@ -181,7 +229,9 @@ var gBrowserInit = {
   },
 
   onLoad() {
-    gBrowser.addEventListener("DOMUpdateBlockedPopups", gPopupBlockerObserver);
+    gBrowser.addEventListener("DOMUpdateBlockedPopups", e =>
+      PopupBlockerObserver.handleEvent(e)
+    );
     gBrowser.addEventListener(
       "TranslationsParent:LanguageState",
       FullPageTranslationsPanel
@@ -316,10 +366,6 @@ var gBrowserInit = {
 
     this._cancelDelayedStartup();
 
-    // Bug 1531854 - The hidden window is force-created here
-    // until all of its dependencies are handled.
-    Services.appShell.hiddenDOMWindow;
-
     gBrowser.addEventListener(
       "PermissionStateChange",
       function () {
@@ -369,22 +415,23 @@ var gBrowserInit = {
     BrowserOffline.init();
     CanvasPermissionPromptHelper.init();
     WebAuthnPromptHelper.init();
-    ContentAnalysis.initialize(document);
+
+    BrowserUtils.callModulesFromCategory(
+      "browser-window-delayed-startup",
+      window
+    );
 
     // Initialize the full zoom setting.
     // We do this before the session restore service gets initialized so we can
     // apply full zoom settings to tabs restored by the session restore service.
     FullZoom.init();
     PanelUI.init(shouldSuppressPopupNotifications);
-    ReportBrokenSite.init(gBrowser);
 
     UpdateUrlbarSearchSplitterState();
 
     BookmarkingUI.init();
     BrowserSearch.delayedStartupInit();
-    SearchUIUtils.init();
     gProtectionsHandler.init();
-    HomePage.delayedStartup().catch(console.error);
 
     let safeMode = document.getElementById("helpSafeMode");
     if (Services.appinfo.inSafeMode) {
@@ -606,6 +653,10 @@ var gBrowserInit = {
 
     ShoppingSidebarManager.ensureInitialized();
 
+    if (Services.prefs.getBoolPref("browser.profiles.enabled", false)) {
+      SelectableProfileService?.init();
+    }
+
     SessionStore.promiseAllWindowsRestored.then(() => {
       this._schedulePerWindowIdleTasks();
       document.documentElement.setAttribute("sessionrestored", "true");
@@ -883,7 +934,6 @@ var gBrowserInit = {
               "resource:///modules/DownloadsMacFinderProgress.sys.mjs"
             ).DownloadsMacFinderProgress.register();
           }
-          Services.telemetry.setEventRecordingEnabled("downloads", true);
         } catch (ex) {
           console.error(ex);
         }
@@ -903,6 +953,10 @@ var gBrowserInit = {
       gGfxUtils.init();
     });
 
+    scheduleIdleTask(async () => {
+      await gProfiles.init();
+    });
+
     // This should always go last, since the idle tasks (except for the ones with
     // timeouts) should execute in order. Note that this observer notification is
     // not guaranteed to fire, since the window could close before we get here.
@@ -913,10 +967,6 @@ var gBrowserInit = {
         "browser-idle-startup-tasks-finished"
       );
     });
-
-    scheduleIdleTask(() => {
-      gProfiles.init();
-    });
   },
 
   // Returns the URI(s) to load at startup if it is immediately known, or a
@@ -1017,8 +1067,6 @@ var gBrowserInit = {
 
     TabletModeUpdater.uninit();
 
-    gTabletModePageCounter.finish();
-
     CaptivePortalWatcher.uninit();
 
     SidebarController.uninit();
@@ -1035,6 +1083,8 @@ var gBrowserInit = {
 
     FirefoxViewHandler.uninit();
 
+    SelectableProfileService?.uninit();
+
     // Now either cancel delayedStartup, or clean up the services initialized from
     // it.
     if (this._boundDelayedStartup) {
@@ -1103,5 +1153,3 @@ var gBrowserInit = {
     window.browserDOMWindow = null;
   },
 };
-
-gBrowserInit.idleTasksFinishedPromise = gBrowserInit.idleTasksFinished.promise;
