@@ -6,6 +6,7 @@ import { DSCard, PlaceholderDSCard } from "../DSCard/DSCard.jsx";
 import { DSEmptyState } from "../DSEmptyState/DSEmptyState.jsx";
 import { DSDismiss } from "content-src/components/DiscoveryStreamComponents/DSDismiss/DSDismiss";
 import { TopicsWidget } from "../TopicsWidget/TopicsWidget.jsx";
+import { ListFeed } from "../ListFeed/ListFeed.jsx";
 import { SafeAnchor } from "../SafeAnchor/SafeAnchor";
 import { FluentOrText } from "../../FluentOrText/FluentOrText.jsx";
 import { actionCreators as ac, actionTypes as at } from "common/Actions.mjs";
@@ -13,6 +14,17 @@ import React, { useEffect, useState, useRef, useCallback } from "react";
 import { connect, useSelector } from "react-redux";
 const PREF_ONBOARDING_EXPERIENCE_DISMISSED =
   "discoverystream.onboardingExperience.dismissed";
+const PREF_THUMBS_UP_DOWN_ENABLED = "discoverystream.thumbsUpDown.enabled";
+const PREF_TOPICS_ENABLED = "discoverystream.topicLabels.enabled";
+const PREF_TOPICS_SELECTED = "discoverystream.topicSelection.selectedTopics";
+const PREF_TOPICS_AVAILABLE = "discoverystream.topicSelection.topics";
+const PREF_SPOCS_STARTUPCACHE_ENABLED =
+  "discoverystream.spocs.startupCache.enabled";
+const PREF_LIST_FEED_ENABLED = "discoverystream.contextualContent.enabled";
+const PREF_LIST_FEED_SELECTED_FEED =
+  "discoverystream.contextualContent.selectedFeed";
+const PREF_FAKESPOT_ENABLED =
+  "discoverystream.contextualContent.fakespot.enabled";
 const INTERSECTION_RATIO = 0.5;
 const VISIBLE = "visible";
 const VISIBILITY_CHANGE_EVENT = "visibilitychange";
@@ -141,6 +153,7 @@ export function OnboardingExperience({ dispatch, windowObj = globalThis }) {
   );
 }
 
+// eslint-disable-next-line no-shadow
 export function IntersectionObserver({
   children,
   windowObj = window,
@@ -307,14 +320,12 @@ export function RecentSavesContainer({
 }
 
 export class _CardGrid extends React.PureComponent {
+  // eslint-disable-next-line max-statements
   renderCards() {
     const prefs = this.props.Prefs.values;
     const {
       items,
-      hybridLayout,
-      hideCardBackground,
       fourCardLayout,
-      compactGrid,
       essentialReadsHeader,
       editorsPicksHeader,
       onboardingExperience,
@@ -323,23 +334,38 @@ export class _CardGrid extends React.PureComponent {
       spocMessageVariant,
       widgets,
       recentSavesEnabled,
-      hideDescriptions,
       DiscoveryStream,
     } = this.props;
-    const { saveToPocketCard } = DiscoveryStream;
+
+    const { saveToPocketCard, topicsLoading } = DiscoveryStream;
     const showRecentSaves = prefs.showRecentSaves && recentSavesEnabled;
     const isOnboardingExperienceDismissed =
       prefs[PREF_ONBOARDING_EXPERIENCE_DISMISSED];
-
-    const recs = this.props.data.recommendations.slice(0, items);
+    const mayHaveThumbsUpDown = prefs[PREF_THUMBS_UP_DOWN_ENABLED];
+    const showTopics = prefs[PREF_TOPICS_ENABLED];
+    const selectedTopics = prefs[PREF_TOPICS_SELECTED];
+    const availableTopics = prefs[PREF_TOPICS_AVAILABLE];
+    const spocsStartupCacheEnabled = prefs[PREF_SPOCS_STARTUPCACHE_ENABLED];
+    const listFeedEnabled = prefs[PREF_LIST_FEED_ENABLED];
+    const listFeedSelectedFeed = prefs[PREF_LIST_FEED_SELECTED_FEED];
+    // filter out recs that should be in ListFeed
+    const recs = this.props.data.recommendations
+      .filter(item => !item.feedName)
+      .slice(0, items);
     const cards = [];
+
     let essentialReadsCards = [];
     let editorsPicksCards = [];
 
     for (let index = 0; index < items; index++) {
       const rec = recs[index];
       cards.push(
-        !rec || rec.placeholder ? (
+        topicsLoading ||
+          !rec ||
+          rec.placeholder ||
+          (rec.flight_id &&
+            !spocsStartupCacheEnabled &&
+            this.props.App.isForStartupCache) ? (
           <PlaceholderDSCard key={`dscard-${index}`} />
         ) : (
           <DSCard
@@ -351,6 +377,10 @@ export class _CardGrid extends React.PureComponent {
             word_count={rec.word_count}
             time_to_read={rec.time_to_read}
             title={rec.title}
+            topic={rec.topic}
+            showTopics={showTopics}
+            selectedTopics={selectedTopics}
+            availableTopics={availableTopics}
             excerpt={rec.excerpt}
             url={rec.url}
             id={rec.id}
@@ -373,6 +403,12 @@ export class _CardGrid extends React.PureComponent {
             spocMessageVariant={spocMessageVariant}
             recommendation_id={rec.recommendation_id}
             firstVisibleTimestamp={this.props.firstVisibleTimestamp}
+            mayHaveThumbsUpDown={mayHaveThumbsUpDown}
+            scheduled_corpus_item_id={rec.scheduled_corpus_item_id}
+            recommended_at={rec.recommended_at}
+            received_rank={rec.received_rank}
+            format={rec.format}
+            alt_text={rec.alt_text}
           />
         )
       );
@@ -412,6 +448,21 @@ export class _CardGrid extends React.PureComponent {
         }
       }
     }
+    if (listFeedEnabled) {
+      const isFakespot = listFeedSelectedFeed === "fakespot";
+      const fakespotEnabled = prefs[PREF_FAKESPOT_ENABLED];
+      if (!isFakespot || (isFakespot && fakespotEnabled)) {
+        // Place the list feed as the 3rd element in the card grid
+        cards.splice(
+          2,
+          1,
+          this.renderListFeed(
+            this.props.data.recommendations,
+            listFeedSelectedFeed
+          )
+        );
+      }
+    }
 
     let moreRecsHeader = "";
     // For now this is English only.
@@ -431,21 +482,7 @@ export class _CardGrid extends React.PureComponent {
       }
     }
 
-    const hideCardBackgroundClass = hideCardBackground
-      ? `ds-card-grid-hide-background`
-      : ``;
-    const fourCardLayoutClass = fourCardLayout
-      ? `ds-card-grid-four-card-variant`
-      : ``;
-    const hideDescriptionsClassName = !hideDescriptions
-      ? `ds-card-grid-include-descriptions`
-      : ``;
-    const compactGridClassName = compactGrid ? `ds-card-grid-compact` : ``;
-    const hybridLayoutClassName = hybridLayout
-      ? `ds-card-grid-hybrid-layout`
-      : ``;
-
-    const gridClassName = `ds-card-grid ${hybridLayoutClassName} ${hideCardBackgroundClass} ${fourCardLayoutClass} ${hideDescriptionsClassName} ${compactGridClassName}`;
+    const gridClassName = this.renderGridClassName();
 
     return (
       <>
@@ -487,6 +524,65 @@ export class _CardGrid extends React.PureComponent {
     );
   }
 
+  renderListFeed(recommendations, selectedFeed) {
+    const recs = recommendations.filter(item => item.feedName === selectedFeed);
+    const isFakespot = selectedFeed === "fakespot";
+    // remove duplicates from category list
+    const categories = [...new Set(recs.map(({ category }) => category))];
+    const listFeed = (
+      <ListFeed
+        // only display recs that match selectedFeed for ListFeed
+        recs={recs}
+        categories={isFakespot ? categories : []}
+        firstVisibleTimestamp={this.props.firstVisibleTimestamp}
+        type={this.props.type}
+        dispatch={this.props.dispatch}
+      />
+    );
+    return listFeed;
+  }
+
+  renderGridClassName() {
+    const prefs = this.props.Prefs.values;
+    const {
+      hybridLayout,
+      hideCardBackground,
+      fourCardLayout,
+      compactGrid,
+      hideDescriptions,
+    } = this.props;
+
+    const adSizingVariantAEnabled = prefs["newtabAdSize.variant-a"];
+    const adSizingVariantBEnabled = prefs["newtabAdSize.variant-b"];
+    const adSizingVariantEnabled =
+      adSizingVariantAEnabled || adSizingVariantBEnabled;
+
+    let adSizingVariantClassName = "";
+    if (adSizingVariantEnabled) {
+      // Ad sizing experiment variant, we want to ensure only 1 of these is ever enabled.
+      adSizingVariantClassName = adSizingVariantAEnabled
+        ? `ad-sizing-variant-a`
+        : `ad-sizing-variant-b`;
+    }
+
+    const hideCardBackgroundClass = hideCardBackground
+      ? `ds-card-grid-hide-background`
+      : ``;
+    const fourCardLayoutClass = fourCardLayout
+      ? `ds-card-grid-four-card-variant`
+      : ``;
+    const hideDescriptionsClassName = !hideDescriptions
+      ? `ds-card-grid-include-descriptions`
+      : ``;
+    const compactGridClassName = compactGrid ? `ds-card-grid-compact` : ``;
+    const hybridLayoutClassName = hybridLayout
+      ? `ds-card-grid-hybrid-layout`
+      : ``;
+
+    const gridClassName = `ds-card-grid ${hybridLayoutClassName} ${hideCardBackgroundClass} ${fourCardLayoutClass} ${hideDescriptionsClassName} ${compactGridClassName} ${adSizingVariantClassName}`;
+    return gridClassName;
+  }
+
   render() {
     const { data } = this.props;
 
@@ -532,5 +628,6 @@ _CardGrid.defaultProps = {
 
 export const CardGrid = connect(state => ({
   Prefs: state.Prefs,
+  App: state.App,
   DiscoveryStream: state.DiscoveryStream,
 }))(_CardGrid);
