@@ -320,11 +320,12 @@ add_task(async function testSourceTreeOnTheIntegrationTestPage() {
     .getAllThreads()
     .find(thread => thread.name == "Main Thread");
 
+  // When EFT is disabled the iframe's source is meld into the main target.
+  const expectedSameUrlSources = isEveryFrameTargetEnabled() ? 3 : 4;
   is(
     sourceActors.filter(actor => actor.thread == mainThread.actor).length,
-    // When EFT is disabled the iframe's source is meld into the main target
-    isEveryFrameTargetEnabled() ? 3 : 4,
-    "same-url.js is loaded 3 times in the main thread"
+    expectedSameUrlSources,
+    `same-url.js is loaded ${expectedSameUrlSources} times in the main thread`
   );
 
   if (isEveryFrameTargetEnabled()) {
@@ -364,8 +365,17 @@ add_task(async function testSourceTreeOnTheIntegrationTestPage() {
   // Framework icons are only displayed when we parse the source,
   // which happens when we select the source
   assertSourceIcon(dbg, "react-component-module.js", "javascript");
-  await selectSource(dbg, "react-component-module.js");
+  const onResumed = SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [],
+    function () {
+      content.eval("pauseInReact()");
+    }
+  );
+  await waitForPaused(dbg);
   assertSourceIcon(dbg, "react-component-module.js", "react");
+  await resume(dbg);
+  await onResumed;
 
   info("Verify blackbox source icon");
   await selectSource(dbg, "script.js");
@@ -436,18 +446,23 @@ add_task(async function testSourceTreeOnTheIntegrationTestPage() {
 add_task(async function testSourceTreeWithWebExtensionContentScript() {
   const extension = await installAndStartContentScriptExtension();
 
-  info("Without the chrome preference, the content script doesn't show up");
-  await pushPref("devtools.chrome.enabled", false);
+  // Ensure that the setting to show content script is off before running the test
+  await pushPref("devtools.debugger.show-content-scripts", false);
   let dbg = await initDebugger("doc-content-script-sources.html");
   // Let some time for unexpected source to appear
   await wait(1000);
   await waitForSourcesInSourceTree(dbg, []);
   await dbg.toolbox.closeToolbox();
 
-  info("With the chrome preference, the content script shows up");
-  await pushPref("devtools.chrome.enabled", true);
   const toolbox = await openToolboxForTab(gBrowser.selectedTab, "jsdebugger");
   dbg = createDebuggerContext(toolbox);
+
+  info("Enable the content script setting");
+  await toggleSourcesTreeSettingsMenuItem(dbg, {
+    className: ".debugger-settings-menu-item-show-content-scripts",
+    isChecked: false,
+  });
+
   await waitForSourcesInSourceTree(dbg, ["content_script.js"]);
   await selectSource(dbg, "content_script.js");
   ok(
