@@ -52,14 +52,14 @@ ChromeUtils.defineESModuleGetters(lazy, {
 
 const HTML_NS = "http://www.w3.org/1999/xhtml";
 
-const SHARED_SWATCH_CLASS = "ruleview-swatch";
-const COLOR_SWATCH_CLASS = "ruleview-colorswatch";
-const BEZIER_SWATCH_CLASS = "ruleview-bezierswatch";
-const LINEAR_EASING_SWATCH_CLASS = "ruleview-lineareasingswatch";
-const FILTER_SWATCH_CLASS = "ruleview-filterswatch";
-const ANGLE_SWATCH_CLASS = "ruleview-angleswatch";
+const SHARED_SWATCH_CLASS = "inspector-swatch";
+const COLOR_SWATCH_CLASS = "inspector-colorswatch";
+const BEZIER_SWATCH_CLASS = "inspector-bezierswatch";
+const LINEAR_EASING_SWATCH_CLASS = "inspector-lineareasingswatch";
+const FILTER_SWATCH_CLASS = "inspector-filterswatch";
+const ANGLE_SWATCH_CLASS = "inspector-angleswatch";
 const FONT_FAMILY_CLASS = "ruleview-font-family";
-const SHAPE_SWATCH_CLASS = "ruleview-shapeswatch";
+const SHAPE_SWATCH_CLASS = "inspector-shapeswatch";
 
 /*
  * An actionable element is an element which on click triggers a specific action
@@ -133,6 +133,7 @@ function TextPropertyEditor(ruleEditor, property) {
   this.telemetry = this.toolbox.telemetry;
 
   this._isDragging = false;
+  this._capturingPointerId = null;
   this._hasDragged = false;
   this._draggingController = null;
   this._draggingValueCache = null;
@@ -153,9 +154,9 @@ function TextPropertyEditor(ruleEditor, property) {
   this._onValidate = this.ruleView.debounce(this._previewValue, 10, this);
   this._onValueDone = this._onValueDone.bind(this);
 
-  this._draggingOnMouseDown = this._draggingOnMouseDown.bind(this);
+  this._draggingOnPointerDown = this._draggingOnPointerDown.bind(this);
   this._draggingOnMouseMove = throttle(this._draggingOnMouseMove, 30, this);
-  this._draggingOnMouseUp = this._draggingOnMouseUp.bind(this);
+  this._draggingOnPointerUp = this._draggingOnPointerUp.bind(this);
   this._draggingOnKeydown = this._draggingOnKeydown.bind(this);
 
   this._create();
@@ -323,9 +324,8 @@ TextPropertyEditor.prototype = {
         }
       });
 
-      const cssVariables = this.rule.elementStyle.getAllCustomProperties(
-        this.rule.pseudoElement
-      );
+      const getCssVariables = () =>
+        this.rule.elementStyle.getAllCustomProperties(this.rule.pseudoElement);
 
       editableField({
         start: this._onStartEditing,
@@ -336,7 +336,7 @@ TextPropertyEditor.prototype = {
         contentType: InplaceEditor.CONTENT_TYPES.CSS_PROPERTY,
         popup: this.popup,
         cssProperties: this.cssProperties,
-        cssVariables,
+        getCssVariables,
         // (Shift+)Tab will move the focus to the previous/next editable field (so property value
         // or new selector).
         focusEditableFieldAfterApply: true,
@@ -394,8 +394,8 @@ TextPropertyEditor.prototype = {
         }
       });
 
-      this.valueSpan.addEventListener("mouseup", () => {
-        // if we have dragged, we will handle the pending click in _draggingOnMouseUp instead
+      this.valueSpan.addEventListener("pointerup", () => {
+        // if we have dragged, we will handle the pending click in _draggingOnPointerUp instead
         if (this._hasDragged) {
           return;
         }
@@ -435,7 +435,7 @@ TextPropertyEditor.prototype = {
         multiline: true,
         maxWidth: () => this.container.getBoundingClientRect().width,
         cssProperties: this.cssProperties,
-        cssVariables,
+        getCssVariables,
         getGridLineNames: this.getGridlineNames,
         showSuggestCompletionOnEmpty: true,
         // (Shift+)Tab will move the focus to the previous/next editable field (so property name,
@@ -581,7 +581,7 @@ TextPropertyEditor.prototype = {
     }
 
     const outputParser = this.ruleView._outputParser;
-    const parserOptions = {
+    this.outputParserOptions = {
       angleClass: "ruleview-angle",
       angleSwatchClass: SHARED_SWATCH_CLASS + " " + ANGLE_SWATCH_CLASS,
       bezierClass: "ruleview-bezier",
@@ -590,12 +590,12 @@ TextPropertyEditor.prototype = {
       colorSwatchClass: SHARED_SWATCH_CLASS + " " + COLOR_SWATCH_CLASS,
       filterClass: "ruleview-filter",
       filterSwatchClass: SHARED_SWATCH_CLASS + " " + FILTER_SWATCH_CLASS,
-      flexClass: "ruleview-flex js-toggle-flexbox-highlighter",
-      gridClass: "ruleview-grid js-toggle-grid-highlighter",
+      flexClass: "inspector-flex js-toggle-flexbox-highlighter",
+      gridClass: "inspector-grid js-toggle-grid-highlighter",
       linearEasingClass: "ruleview-lineareasing",
       linearEasingSwatchClass:
         SHARED_SWATCH_CLASS + " " + LINEAR_EASING_SWATCH_CLASS,
-      shapeClass: "ruleview-shape",
+      shapeClass: "inspector-shape",
       shapeSwatchClass: SHAPE_SWATCH_CLASS,
       // Only ask the parser to convert colors to the default color type specified by the
       // user if the property hasn't been changed yet.
@@ -604,15 +604,24 @@ TextPropertyEditor.prototype = {
       urlClass: "theme-link",
       fontFamilyClass: FONT_FAMILY_CLASS,
       baseURI: this.sheetHref,
-      unmatchedVariableClass: "ruleview-unmatched-variable",
-      matchedVariableClass: "ruleview-variable",
+      unmatchedClass: "inspector-unmatched",
+      matchedVariableClass: "inspector-variable",
       getVariableData: varName =>
         this.rule.elementStyle.getVariableData(
           varName,
           this.rule.pseudoElement
         ),
+      inStartingStyleRule: this.rule.isInStartingStyle(),
     };
-    const frag = outputParser.parseCssProperty(name, val, parserOptions);
+
+    if (this.rule.darkColorScheme !== undefined) {
+      this.outputParserOptions.isDarkColorScheme = this.rule.darkColorScheme;
+    }
+    const frag = outputParser.parseCssProperty(
+      name,
+      val,
+      this.outputParserOptions
+    );
 
     // Save the initial value as the last committed value,
     // for restoring after pressing escape.
@@ -750,7 +759,7 @@ TextPropertyEditor.prototype = {
     const span = this.valueSpan.querySelector("." + FILTER_SWATCH_CLASS);
     if (this.ruleEditor.isEditable) {
       if (span) {
-        parserOptions.filterSwatch = true;
+        this.outputParserOptions.filterSwatch = true;
 
         this.ruleView.tooltips.getTooltip("filterEditor").addSwatch(
           span,
@@ -761,7 +770,7 @@ TextPropertyEditor.prototype = {
             onRevert: this._onSwatchRevert,
           },
           outputParser,
-          parserOptions
+          this.outputParserOptions
         );
         const title = l10n("rule.filterSwatch.tooltip");
         span.setAttribute("title", title);
@@ -781,22 +790,22 @@ TextPropertyEditor.prototype = {
 
     const nodeFront = this.ruleView.inspector.selection.nodeFront;
 
-    const flexToggle = this.valueSpan.querySelector(".ruleview-flex");
+    const flexToggle = this.valueSpan.querySelector(".inspector-flex");
     if (flexToggle) {
       flexToggle.setAttribute("title", l10n("rule.flexToggle.tooltip"));
-      flexToggle.classList.toggle(
-        "active",
+      flexToggle.setAttribute(
+        "aria-pressed",
         this.ruleView.inspector.highlighters.getNodeForActiveHighlighter(
           this.ruleView.inspector.highlighters.TYPES.FLEXBOX
         ) === nodeFront
       );
     }
 
-    const gridToggle = this.valueSpan.querySelector(".ruleview-grid");
+    const gridToggle = this.valueSpan.querySelector(".inspector-grid");
     if (gridToggle) {
       gridToggle.setAttribute("title", l10n("rule.gridToggle.tooltip"));
-      gridToggle.classList.toggle(
-        "active",
+      gridToggle.setAttribute(
+        "aria-pressed",
         this.ruleView.highlighters.gridHighlighters.has(nodeFront)
       );
       gridToggle.toggleAttribute(
@@ -805,7 +814,7 @@ TextPropertyEditor.prototype = {
       );
     }
 
-    const shapeToggle = this.valueSpan.querySelector(".ruleview-shapeswatch");
+    const shapeToggle = this.valueSpan.querySelector(".inspector-shapeswatch");
     if (shapeToggle) {
       const mode =
         "css" +
@@ -816,6 +825,8 @@ TextPropertyEditor.prototype = {
           })
           .join("");
       shapeToggle.setAttribute("data-mode", mode);
+      shapeToggle.setAttribute("aria-pressed", false);
+      shapeToggle.setAttribute("title", l10n("rule.shapeToggle.tooltip"));
     }
 
     // Now that we have updated the property's value, we might have a pending
@@ -1057,7 +1068,7 @@ TextPropertyEditor.prototype = {
 
     const outputParser = this.ruleView._outputParser;
     const frag = outputParser.parseCssProperty(computed.name, computed.value, {
-      colorSwatchClass: "ruleview-swatch ruleview-colorswatch",
+      colorSwatchClass: "inspector-swatch inspector-colorswatch",
       urlClass: "theme-link",
       baseURI: this.sheetHref,
       fontFamilyClass: "ruleview-font-family",
@@ -1185,9 +1196,16 @@ TextPropertyEditor.prototype = {
       return;
     }
 
-    // Remove a property if the property value is empty and the property
-    // value is not about to be focused
-    if (!this.prop.value && direction !== Services.focus.MOVEFOCUS_FORWARD) {
+    const isVariable = value.startsWith("--");
+
+    // Remove a property if:
+    // - the property value is empty and is not a variable (empty variables are valid)
+    // - and the property value is not about to be focused
+    if (
+      !this.prop.value &&
+      !isVariable &&
+      direction !== Services.focus.MOVEFOCUS_FORWARD
+    ) {
       this.remove(direction);
       return;
     }
@@ -1267,9 +1285,11 @@ TextPropertyEditor.prototype = {
         this.committed.value === val.value &&
         this.committed.priority === val.priority);
 
-    // If the value is not empty and unchanged, revert the property back to
-    // its original value and enabled or disabled state
-    if (value.trim() && isValueUnchanged) {
+    const isVariable = this.prop.name.startsWith("--");
+
+    // If the value is not empty (or is an empty variable) and unchanged,
+    // revert the property back to its original value and enabled or disabled state
+    if ((value.trim() || isVariable) && isValueUnchanged) {
       this.ruleEditor.rule.previewPropertyValue(
         this.prop,
         val.value,
@@ -1301,12 +1321,17 @@ TextPropertyEditor.prototype = {
     // If needed, add any new properties after this.prop.
     this.ruleEditor.addProperties(parsedProperties.propertiesToAdd, this.prop);
 
-    // If the input value is empty and the focus is moving forward to the next
-    // editable field, then remove the whole property.
+    // If the input value is empty and is not a variable (empty variables are valid),
+    // and the focus is moving forward to the next editable field,
+    // then remove the whole property.
     // A timeout is used here to accurately check the state, since the inplace
     // editor `done` and `destroy` events fire before the next editor
     // is focused.
-    if (!value.trim() && direction !== Services.focus.MOVEFOCUS_BACKWARD) {
+    if (
+      !value.trim() &&
+      !isVariable &&
+      direction !== Services.focus.MOVEFOCUS_BACKWARD
+    ) {
       setTimeout(() => {
         if (!this.editing) {
           this.remove(direction);
@@ -1466,9 +1491,16 @@ TextPropertyEditor.prototype = {
     return !!dimensionMatchObj;
   },
 
-  _draggingOnMouseDown(event) {
+  _draggingOnPointerDown(event) {
+    // We want to handle a drag during a mouse button is pressed.  So, we can
+    // ignore pointer events which are caused by other devices.
+    if (event.pointerType != "mouse") {
+      return;
+    }
+
     this._isDragging = true;
     this.valueSpan.setPointerCapture(event.pointerId);
+    this._capturingPointerId = event.pointerId;
     this._draggingController = new AbortController();
     const { signal } = this._draggingController;
 
@@ -1484,10 +1516,12 @@ TextPropertyEditor.prototype = {
       unit,
     };
 
+    // "pointermove" is fired when the button state is changed too.  Therefore,
+    // we should listen to "mousemove" to handle the pointer position changes.
     this.valueSpan.addEventListener("mousemove", this._draggingOnMouseMove, {
       signal,
     });
-    this.valueSpan.addEventListener("mouseup", this._draggingOnMouseUp, {
+    this.valueSpan.addEventListener("pointerup", this._draggingOnPointerUp, {
       signal,
     });
     this.valueSpan.addEventListener("keydown", this._draggingOnKeydown, {
@@ -1537,12 +1571,13 @@ TextPropertyEditor.prototype = {
     const { value, unit } = this._draggingValueCache;
     // We use toFixed to avoid the case where value is too long, 9.00001px for example
     const roundedValue = Number.isInteger(value) ? value : value.toFixed(1);
-    this.prop.setValue(roundedValue + unit, this.prop.priority);
-    this.ruleView.emitForTests("property-updated-by-dragging");
+    this.prop
+      .setValue(roundedValue + unit, this.prop.priority)
+      .then(() => this.ruleView.emitForTests("property-updated-by-dragging"));
     this._hasDragged = true;
   },
 
-  _draggingOnMouseUp(event) {
+  _draggingOnPointerUp() {
     if (!this._isDragging) {
       return;
     }
@@ -1550,18 +1585,18 @@ TextPropertyEditor.prototype = {
       this.committed.value = this.prop.value;
       this.prop.setEnabled(true);
     }
-    this._onStopDragging(event);
+    this._onStopDragging();
   },
 
   _draggingOnKeydown(event) {
     if (event.key == "Escape") {
       this.prop.setValue(this.committed.value, this.committed.priority);
-      this._onStopDragging(event);
+      this._onStopDragging();
       event.preventDefault();
     }
   },
 
-  _onStopDragging(event) {
+  _onStopDragging() {
     // childHasDragged is used to stop the propagation of a click event when we
     // release the mouse in the ruleview.
     // The click event is not emitted when we have a pending click on the text property.
@@ -1571,7 +1606,15 @@ TextPropertyEditor.prototype = {
     this._isDragging = false;
     this._hasDragged = false;
     this._draggingValueCache = null;
-    this.valueSpan.releasePointerCapture(event.pointerId);
+    if (this._capturingPointerId !== null) {
+      this._capturingPointerId = null;
+      try {
+        this.valueSpan.releasePointerCapture(this._capturingPointerId);
+      } catch (e) {
+        // Ignore exception even if the pointerId has already been invalidated
+        // before the capture has already been released implicitly.
+      }
+    }
     this.valueSpan.classList.remove(IS_DRAGGING_CLASSNAME);
     this._draggingController.abort();
   },
@@ -1585,7 +1628,11 @@ TextPropertyEditor.prototype = {
       return;
     }
     this.valueSpan.classList.add(DRAGGABLE_VALUE_CLASSNAME);
-    this.valueSpan.addEventListener("mousedown", this._draggingOnMouseDown);
+    this.valueSpan.addEventListener(
+      "pointerdown",
+      this._draggingOnPointerDown,
+      { passive: true }
+    );
   },
 
   _removeDraggingCapacity() {
@@ -1594,7 +1641,11 @@ TextPropertyEditor.prototype = {
     }
     this._draggingController = null;
     this.valueSpan.classList.remove(DRAGGABLE_VALUE_CLASSNAME);
-    this.valueSpan.removeEventListener("mousedown", this._draggingOnMouseDown);
+    this.valueSpan.removeEventListener(
+      "pointerdown",
+      this._draggingOnPointerDown,
+      { passive: true }
+    );
   },
 
   /**
