@@ -474,6 +474,10 @@ class WindowGlobalTargetActor extends BaseTargetActor {
       : null;
   }
 
+  get targetGlobal() {
+    return this.window;
+  }
+
   get outerWindowID() {
     if (this.docShell) {
       return this.docShell.outerWindowID;
@@ -583,8 +587,13 @@ class WindowGlobalTargetActor extends BaseTargetActor {
 
   /**
    * Getter for the window global's title.
+   * For Web Extension it will ignore the document title and refer to the addon one.
    */
   get title() {
+    if (this.sessionContext.type == "webextension") {
+      const policy = WebExtensionPolicy.getByID(this.sessionContext.addonId);
+      return policy.name;
+    }
     return this.contentDocument.title;
   }
 
@@ -743,6 +752,12 @@ class WindowGlobalTargetActor extends BaseTargetActor {
     }
     this.destroying = true;
 
+    // Force flushing pending resources if the actor isn't already destroyed.
+    // This helps notify the client about pending resources on navigation.
+    if (!this.isDestroyed()) {
+      this.emitResources();
+    }
+
     // Tell the thread actor that the window global is closed, so that it may terminate
     // instead of resuming the debuggee script.
     // TODO: Bug 997119: Remove this coupling with thread actor
@@ -862,6 +877,13 @@ class WindowGlobalTargetActor extends BaseTargetActor {
       return;
     }
 
+    // This method is called asynchronously and the document may have been destroyed in the meantime.
+    // In such case, automatically destroy the target actor.
+    if (this.docShell.isBeingDestroyed()) {
+      this.destroy();
+      return;
+    }
+
     // In child processes, we watch all docshells living in the process.
     Services.obs.addObserver(this, "webnavigation-create");
     Services.obs.addObserver(this, "webnavigation-destroy");
@@ -978,7 +1000,6 @@ class WindowGlobalTargetActor extends BaseTargetActor {
     scriptError.initWithWindowID(
       text,
       null,
-      null,
       0,
       0,
       flags,
@@ -1737,6 +1758,12 @@ class DebuggerProgressListener {
   }
 
   unwatch(docShell) {
+    // If the docshell is being destroyed, we won't be able to retrieve its related window object,
+    // which is the key ingredient for all cleanup operations done in this method.
+    if (docShell.isBeingDestroyed()) {
+      return;
+    }
+
     const docShellWindow = docShell.domWindow;
     if (!this._watchedDocShells.has(docShellWindow)) {
       return;
