@@ -7,6 +7,7 @@
 #include "ChromeUtils.h"
 
 #include "JSOracleParent.h"
+#include "ThirdPartyUtil.h"
 #include "js/CallAndConstruct.h"  // JS::Call
 #include "js/ColumnNumber.h"  // JS::TaggedColumnNumberOneOrigin, JS::ColumnNumberOneOrigin
 #include "js/CharacterEncoding.h"
@@ -32,6 +33,7 @@
 #include "mozilla/ScopeExit.h"
 #include "mozilla/ScrollingMetrics.h"
 #include "mozilla/SharedStyleSheetCache.h"
+#include "mozilla/dom/SharedScriptCache.h"
 #include "mozilla/SpinEventLoopUntil.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/dom/ContentParent.h"
@@ -57,7 +59,6 @@
 #include "mozilla/RemoteDecoderManagerChild.h"
 #include "mozilla/KeySystemConfig.h"
 #include "mozilla/WheelHandlingHelper.h"
-#include "IOActivityMonitor.h"
 #include "nsNativeTheme.h"
 #include "nsThreadUtils.h"
 #include "mozJSModuleLoader.h"
@@ -187,23 +188,22 @@ void ChromeUtils::ReleaseAssert(GlobalObject& aGlobal, bool aCondition,
   }
 
   // Extract the current stack from the JS runtime to embed in the crash reason.
-  nsAutoString filename;
+  nsAutoCString filename;
   uint32_t lineNo = 0;
 
   if (nsCOMPtr<nsIStackFrame> location = GetCurrentJSStack(1)) {
     location->GetFilename(aGlobal.Context(), filename);
     lineNo = location->GetLineNumber(aGlobal.Context());
   } else {
-    filename.Assign(u"<unknown>"_ns);
+    filename.Assign("<unknown>"_ns);
   }
 
   // Convert to utf-8 for adding as the MozCrashReason.
-  NS_ConvertUTF16toUTF8 filenameUtf8(filename);
   NS_ConvertUTF16toUTF8 messageUtf8(aMessage);
 
   // Actually crash.
   MOZ_CRASH_UNSAFE_PRINTF("Failed ChromeUtils.releaseAssert(\"%s\") @ %s:%u",
-                          messageUtf8.get(), filenameUtf8.get(), lineNo);
+                          messageUtf8.get(), filename.get(), lineNo);
 }
 
 /* static */
@@ -896,14 +896,22 @@ static bool JSLazyGetter(JSContext* aCx, unsigned aArgc, JS::Value* aVp) {
 static bool DefineLazyGetter(JSContext* aCx, JS::Handle<JSObject*> aTarget,
                              JS::Handle<JS::Value> aName,
                              JS::Handle<JSObject*> aLambda) {
-  JS::Rooted<jsid> id(aCx);
+  JS::Rooted<JS::PropertyKey> id(aCx);
   if (!JS_ValueToId(aCx, aName, &id)) {
     return false;
   }
 
+  JS::Rooted<JS::PropertyKey> funId(aCx);
+  if (id.isAtom()) {
+    funId = id;
+  } else {
+    // Don't care int and symbol cases.
+    funId = JS::PropertyKey::NonIntAtom(JS_GetEmptyString(aCx));
+  }
+
   JS::Rooted<JSObject*> getter(
-      aCx, JS_GetFunctionObject(
-               js::NewFunctionByIdWithReserved(aCx, JSLazyGetter, 0, 0, id)));
+      aCx, JS_GetFunctionObject(js::NewFunctionByIdWithReserved(
+               aCx, JSLazyGetter, 0, 0, funId)));
   if (!getter) {
     JS_ReportOutOfMemory(aCx);
     return false;
@@ -1184,6 +1192,7 @@ void ChromeUtils::DefineESModuleGetters(
 /* static */
 void ChromeUtils::GetLibcConstants(const GlobalObject&,
                                    LibcConstants& aConsts) {
+  aConsts.mEPERM.Construct(EPERM);
   aConsts.mEINTR.Construct(EINTR);
   aConsts.mEACCES.Construct(EACCES);
   aConsts.mEAGAIN.Construct(EAGAIN);
@@ -1302,27 +1311,68 @@ void ChromeUtils::GetBaseDomainFromPartitionKey(dom::GlobalObject& aGlobal,
 
 /* static */
 void ChromeUtils::GetPartitionKeyFromURL(dom::GlobalObject& aGlobal,
-                                         const nsAString& aURL,
+                                         const nsAString& aTopLevelUrl,
+                                         const nsAString& aSubresourceUrl,
+                                         const Optional<bool>& aForeignContext,
                                          nsAString& aPartitionKey,
                                          ErrorResult& aRv) {
-  nsCOMPtr<nsIURI> uri;
-  nsresult rv = NS_NewURI(getter_AddRefs(uri), aURL);
-  if (NS_SUCCEEDED(rv) && uri->SchemeIs("chrome")) {
+  nsCOMPtr<nsIURI> topLevelURI;
+  nsresult rv = NS_NewURI(getter_AddRefs(topLevelURI), aTopLevelUrl);
+  if (NS_SUCCEEDED(rv) && topLevelURI->SchemeIs("chrome")) {
     rv = NS_ERROR_FAILURE;
   }
-
   if (NS_WARN_IF(NS_FAILED(rv))) {
     aPartitionKey.Truncate();
     aRv.Throw(rv);
     return;
   }
 
-  mozilla::OriginAttributes attrs;
-  // For now, uses assume the partition key is cross-site.
-  // We will need to not make this assumption to allow access
-  // to same-site partitioned cookies in the cookie extension API.
-  attrs.SetPartitionKey(uri, false);
+  bool foreignResource;
+  bool fallback = false;
+  if (!aSubresourceUrl.IsEmpty()) {
+    nsCOMPtr<nsIURI> resourceURI;
+    rv = NS_NewURI(getter_AddRefs(resourceURI), aSubresourceUrl);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      aPartitionKey.Truncate();
+      aRv.Throw(rv);
+      return;
+    }
 
+    ThirdPartyUtil* thirdPartyUtil = ThirdPartyUtil::GetInstance();
+    if (!thirdPartyUtil) {
+      aPartitionKey.Truncate();
+      aRv.Throw(NS_ERROR_SERVICE_NOT_AVAILABLE);
+      return;
+    }
+
+    rv = thirdPartyUtil->IsThirdPartyURI(topLevelURI, resourceURI,
+                                         &foreignResource);
+    if (NS_FAILED(rv)) {
+      // we fallback to assuming the resource is foreign if there is an error
+      foreignResource = true;
+      fallback = true;
+    }
+  } else {
+    // Assume we have a foreign resource if the resource was not provided
+    foreignResource = true;
+    fallback = true;
+  }
+
+  // aForeignContext is whether or not this is a foreign context.
+  // foreignResource is whether or not the resource is cross-site to the top
+  // level. So we need to validate that a false foreign context doesn't have a
+  // same-site resource. That is impossible!
+  if (aForeignContext.WasPassed() && !aForeignContext.Value() &&
+      foreignResource && !fallback) {
+    aPartitionKey.Truncate();
+    aRv.Throw(nsresult::NS_ERROR_INVALID_ARG);
+    return;
+  }
+
+  bool foreignByAncestorContext = aForeignContext.WasPassed() &&
+                                  aForeignContext.Value() && !foreignResource;
+  mozilla::OriginAttributes attrs;
+  attrs.SetPartitionKey(topLevelURI, foreignByAncestorContext);
   aPartitionKey = attrs.mPartitionKey;
 }
 
@@ -1353,18 +1403,36 @@ void ChromeUtils::ClearRecentJSDevError(GlobalObject&) {
 
 void ChromeUtils::ClearStyleSheetCacheByPrincipal(GlobalObject&,
                                                   nsIPrincipal* aForPrincipal) {
-  SharedStyleSheetCache::Clear(aForPrincipal);
+  SharedStyleSheetCache::Clear(Some(aForPrincipal));
 }
 
-void ChromeUtils::ClearStyleSheetCacheByBaseDomain(
-    GlobalObject&, const nsACString& aBaseDomain) {
-  SharedStyleSheetCache::Clear(nullptr, &aBaseDomain);
+void ChromeUtils::ClearStyleSheetCacheBySite(
+    GlobalObject&, const nsACString& aSchemelessSite,
+    const dom::OriginAttributesPatternDictionary& aPattern) {
+  SharedStyleSheetCache::Clear(Nothing(), Some(nsCString(aSchemelessSite)),
+                               Some(OriginAttributesPattern(aPattern)));
 }
 
 void ChromeUtils::ClearStyleSheetCache(GlobalObject&) {
   SharedStyleSheetCache::Clear();
 }
 
+void ChromeUtils::ClearScriptCacheByPrincipal(GlobalObject&,
+                                              nsIPrincipal* aForPrincipal) {
+  SharedScriptCache::Clear(Some(aForPrincipal));
+}
+
+void ChromeUtils::ClearScriptCacheBySite(
+    GlobalObject&, const nsACString& aSchemelessSite,
+    const dom::OriginAttributesPatternDictionary& aPattern) {
+  SharedScriptCache::Clear(Nothing(), Some(nsCString(aSchemelessSite)),
+                           Some(aPattern));
+}
+
+void ChromeUtils::ClearScriptCache(GlobalObject&) {
+  SharedScriptCache::Clear();
+}
+
 #define PROCTYPE_TO_WEBIDL_CASE(_procType, _webidl) \
   case mozilla::ProcType::_procType:                \
     return WebIDLProcType::_webidl
@@ -1388,6 +1456,7 @@ static WebIDLProcType ProcTypeToWebIDL(mozilla::ProcType aType) {
     PROCTYPE_TO_WEBIDL_CASE(PrivilegedMozilla, Privilegedmozilla);
     PROCTYPE_TO_WEBIDL_CASE(WebCOOPCOEP, WithCoopCoep);
     PROCTYPE_TO_WEBIDL_CASE(WebServiceWorker, WebServiceWorker);
+    PROCTYPE_TO_WEBIDL_CASE(Inference, Inference);
 
 #define GECKO_PROCESS_TYPE(enum_value, enum_name, string_name, proc_typename, \
                            process_bin_type, procinfo_typename,               \
@@ -1572,6 +1641,8 @@ already_AddRefed<Promise> ChromeUtils::RequestProcInfo(GlobalObject& aGlobal,
       type = mozilla::ProcType::PrivilegedMozilla;
     } else if (remoteType == PREALLOC_REMOTE_TYPE) {
       type = mozilla::ProcType::Preallocated;
+    } else if (remoteType == INFERENCE_REMOTE_TYPE) {
+      type = mozilla::ProcType::Inference;
     } else if (StringBeginsWith(remoteType, DEFAULT_REMOTE_TYPE)) {
       type = mozilla::ProcType::Web;
     } else {
@@ -1843,22 +1914,6 @@ void ChromeUtils::CreateError(const GlobalObject& aGlobal,
   aRetVal.set(retVal);
 }
 
-/* static */
-already_AddRefed<Promise> ChromeUtils::RequestIOActivity(GlobalObject& aGlobal,
-                                                         ErrorResult& aRv) {
-  MOZ_ASSERT(XRE_IsParentProcess());
-  MOZ_ASSERT(Preferences::GetBool(IO_ACTIVITY_ENABLED_PREF, false));
-  nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(aGlobal.GetAsSupports());
-  MOZ_ASSERT(global);
-  RefPtr<Promise> domPromise = Promise::Create(global, aRv);
-  if (NS_WARN_IF(aRv.Failed())) {
-    return nullptr;
-  }
-  MOZ_ASSERT(domPromise);
-  mozilla::net::IOActivityMonitor::RequestActivities(domPromise);
-  return domPromise.forget();
-}
-
 /* static */
 bool ChromeUtils::HasReportingHeaderForOrigin(GlobalObject& global,
                                               const nsAString& aOrigin,
@@ -2098,6 +2153,9 @@ bool ChromeUtils::ShouldResistFingerprinting(
     case JSRFPTarget::SiteSpecificZoom:
       target = RFPTarget::SiteSpecificZoom;
       break;
+    case JSRFPTarget::CSSPrefersColorScheme:
+      target = RFPTarget::CSSPrefersColorScheme;
+      break;
     default:
       MOZ_CRASH("Unhandled JSRFPTarget enum value");
   }
